/* ----------------------------------------------------------------------
 * getseq -- get quickly a fragment of a sequence
 * Copyright (C) 2000 January Weiner III
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 ---------------------------------------------------------------------- */

#include <stdio.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>

#include "genpak.h"
#include "gp_getopt.h"

#define VERSION "0.6"
#define PROGNAME "gp_getseq"

char *progname ;

/*
 *
 */

typedef struct {
	FILE* in ;
	FILE* out ; } opt_s ;

int readlistfile(FILE *list_fp, long *s, long *e, char* n) ;

int main(int argc, char *argv[])
{
	extern int optind ;
	extern char *optarg ;
	int width=70 ; /* width with which the sequence gets formatted */
	FILE *in, *out, *list_fp ;
	sekw *outseq, *tempseq ;
	int reverse=FALSE, optname=FALSE, list=FALSE, worktodo=TRUE;

	long start=0, end=0;

	int c, sequences_processed=0;
	char optional_name[MAXNAZWA], temp_name[MAXNAZWA] ;
	int errflg = 0 ;

	/* initializing structure holding all warnings */

	allwarnings=NULL ;

	progname=argv[0] ;

	while ((c = gp_getopt(argc, argv, "l:n:rvhqd")) != EOF)
		switch(c) {
		case 'n':
			Warn("Optional sequence name: %s",optarg) ;
			if(strlen(optarg)>MAXNAZWA) optarg[MAXNAZWA]='\0' ;
			strcpy(optional_name,optarg) ;
			optname=TRUE ;
			break ;
		case 'l':
			if(optarg[0]=='-' && strlen(optarg)==1) {
				Warn("Reading list of sequences from standard input") ;
				list_fp=stdin ;
			} else {
				Warn("Reading list of sequences from file %s",optarg) ;
				list_fp=TryOpen(optarg,"r") ;
				if(list_fp==NULL) Error("Could not open file %s",optarg) ;
			}
			list=TRUE ;
			break ;
		case 'r':
			Warn("Only reversing the sequence") ;
			reverse=TRUE ;
			break ;
		case 'H':
			html=TRUE ;
			break ;
		case 'q':
			quiet=TRUE ;
			break ;
		case 'v':
			fprintf(stderr,"%s version %s\n",progname,VERSION) ;
			exit(0) ;
			break ;
		case 'd':
			debug=TRUE ;
			Warn("Running in debug mode") ;
			break ;
		case 'h':
			Help() ;
			break ;
		default:
			errflg++ ;
			break;
		}

	

	if(errflg) {
			Error("Type '%s -h' for help",progname) ;
	}

	/*
	 * Unless we are using a list of sequences to cut out, we have to provide
	 * a sequence start and a sequence end.
	 */

	if(!list && !reverse) {
		if(optind > (argc+2))
			Error("I need at least two arguments: start and end") ;
	
		if(sscanf(argv[optind],"%li",&start) != 1 )
			Error("Could not read the start base parameter") ;
	
		optind++ ;
		
		if(sscanf(argv[optind],"%li",&end) != 1 )
			Error("Could not read the end base parameter") ;

		optind++ ;

		if(debug)
			Warn("Reading sequence from %i to %i",start,end) ;
	}
	
	/* open the file pointer to read the sequences 
	 * from: standard input or a file provided? */

	if(optind >= argc) {
		if(debug) Warn("Reading master sequence from standard input") ;
		in=stdin ;
	} else {
		if(debug) Warn("Reading master sequence from file %s", argv[optind]) ;
		in=TryOpen(argv[optind],"r") ;
	}

	/* opening the file pointer to write the output: 
 	 * standard output or file provided? */
	optind++ ;

	if(optind >= argc) {
		if(debug) Warn("Writing sequences to standard output") ;
		out=stdout ;
	} else {
		if(debug) Warn("Writing sequences to file %s", argv[optind],"wb") ;
		out=TryOpen(argv[optind],"wb") ;
	}


	/* 
	 * This is the code that does all the actual work. If entries are taken
	 * from a list, it is executed as long as "workto" is TRUE. Otherwise
	 * execution stops after the first time.
	 */

	/* reverse option defined: only reverse the master sequence */
	if(reverse) {
		if((tempseq=ReadWholeSequence(in)) == NULL) 
			Error("No master sequence found!") ;
		outseq=ReverseSequence(tempseq) ;
		FormatSeq(out,outseq,width) ;
		return EXIT_SUCCESS ;
	}

	/* reading list of fragments from an optional fragments list. */
	if(list) {

		/* reading the "master" sequence from which all the fragments come */
		if((tempseq=ReadWholeSequence(in)) == NULL) 
			Error("No master sequence found!") ;
		temp_name[0]='\0' ;

		/* reading fragments from the fragment list line by line */
		while(readlistfile(list_fp,&start,&end,temp_name) == EXIT_SUCCESS) {
			if(debug)
				Warn("reading sequence from %i to %i",start,end) ;
			outseq=CopySeqFrag(tempseq,start,end) ;
			if(optname==TRUE) 
				strcpy(outseq->name,optional_name) ;
			else 
				if(temp_name[0] != '\0') strcpy(outseq->name,temp_name);

			FormatSeq(out,outseq,width) ;
			temp_name[0]='\0' ;
		}
		return EXIT_SUCCESS ;
	}

	/* the standard behaviour: get a sequence from start to end */
	if(end > start) {
		outseq=ReadSeqFrag(in,start,end,0) ;
	} else {
		tempseq=ReadSeqFrag(in,end,start,0) ;
		outseq=ReverseSequence(tempseq) ;
	}

	FormatSeq(out,outseq,width) ;

	return EXIT_SUCCESS ;

	if(html) PrintAllWarnings(out) ;

	fclose(out) ;
	fclose(in) ;
	return(EXIT_SUCCESS);
}


/* Read sequence list from file */

int readlistfile(FILE *list_fp, long *s, long *e, char* n) {
	
	char bufor[BUFSIZ+1] ;

	bufor[0]='\0' ;
	n[0]='\0' ;

	do {
		if(fgets(bufor,BUFSIZ,list_fp) == NULL) {
			if(debug) Warn("Reading of list file terminated") ;
			return EXIT_FAILURE ;
		}
	} while(bufor[0]=='#') ;
		
	if(debug) Warn("Read line %s from list file",bufor) ;

	
	if(sscanf(bufor,"%li %li %99s",s,e,n)<2) {
		Warn("Could not interpret list file line") ;
		return EXIT_FAILURE ;
	}

	return EXIT_SUCCESS ;

}

/* Standard mesage */

void Help()
{
printf("\n");
printf("%s version %s  - get a sequence fragment",PROGNAME,VERSION);
printf("\n");
printf("  Usage:\n");
printf("     %s [options] start end [input file] [output file]\n",progname);
printf("     %s -r [options] [input file] [output file]\n",progname);
printf("     %s -l name [options] [input file] [output file]\n",progname);
printf("\n");
printf("  Options:\n");
printf("     -r       : only reverse the sequence\n");
printf("     -l name  : take a list of sequences from file [name]\n");
printf("                (use '-' for standard input)\n");
printf("     -n name  : use alternate name for the sequence\n");
printf("     -H       : run in HTML mode (see man page)\n");
printf("     -q       : run in quiet mode\n");
printf("     -v       : print version information & exit\n");
printf("     -h       : print this help screen & exit\n\n");
exit(0);
}


			
