/* ----------------------------------------------------------------------
 * qs -- quick sequence search
 * Copyright (C) 2000 January Weiner III
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 ---------------------------------------------------------------------- */

#include <stdio.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>

#include "genpak.h"
#include "gp_getopt.h"

#define VERSION "0.1"
#define PROGNAME "gp_qs"
#define VARIA 2

int plus=TRUE,  minus=TRUE, degenerate=FALSE ;
int varia=VARIA ;
char *progname ;

int SearchNeedle(FILE *out,sekw* s, sekw* needle) ;

/*
 *
 */

int main(int argc, char *argv[])
{
	extern int optind ;
	extern char *optarg ;

	FILE *in, *out, *query ;
	sekw *inseq ;
	sekw *needle ;

	int c, needle_stdin=FALSE;
	char message[100] ;
	int errflg = 0 ;

	allwarnings=NULL ;

	progname=argv[0] ;

	while ((c = gp_getopt(argc, argv, "nimpqvdh")) != EOF)
		switch(c) {
		case 'n':
			Warn("Allowing not exact binding, %i",varia) ;
			degenerate=TRUE ;
			break ;
		case 'i':
			Warn("Reading query sequences from standard input") ;
			needle_stdin=TRUE ;
			break ;
		case 'm':
			Warn("Searching only on the minus strang") ;
			plus=FALSE ;
			break ;
		case 'p':
			Warn("Searching only on the plus strang") ;
			minus=FALSE ;
			break ;
		case 'q':
			quiet=TRUE ;
			break ;
		case 'v':
			fprintf(stderr,"%s version %s\n",progname,VERSION) ;
			exit(0) ;
			break ;
		case 'H':
			html=TRUE ;
			break ;
		case 'd':
			debug=TRUE ;
			Warn("Running in debug mode") ;
			break ;
		case 'h':
			Help() ;
			break ;
		default:
			errflg++ ;
			break;
		}

	if(optind >= argc) {
		Warn("Not enough parameters given") ;
		errflg++ ;
	}

	if(errflg) {
			sprintf(message,"Type '%s -h' for help",progname) ;
			Error(message) ;
	}

	/* 
	 * if option -i defined, then reading sequences from stdin 
	 * and the database sequence from a file. 
	 * otherwise, try to get the sequences from command line 
	 * and the database sequence either from a file, or stdin
	 */

	if(needle_stdin==FALSE) {
		needle=(sekw*) calloc(1,sizeof(sekw)) ;
		needle->sequ=(char *) calloc(strlen(argv[optind])+1,sizeof(char)) ;
		strcpy(needle->sequ,argv[optind]) ;
		strcpy(needle->name,"Unknown") ;
		optind++ ;
		if(debug) Warn("Query: %s",needle->sequ) ;
	} else {
		query=stdin ;
		Warn("Reading query from stdandard input") ;
	}

	

	/* open the file pointer to read the sequences 
	 * from: standard input or a file provided? */
	if(optind >= argc) {
		if(needle_stdin==TRUE) 
			Error("Cannot read both query and database from standard input!") ;
		in=stdin ;
	} else {
		if(debug) Warn("Reading database from file %s",argv[optind]) ;
		in=TryOpen(argv[optind],"r") ;
		optind++ ;
	}


/* opening the file pointer to write the output: 
 * standard output or file provided? */

	if(optind >= argc) {
		Warn("Writing to standard output") ;
		out=stdout ;
	} else {
		out=TryOpen(argv[optind],"wb") ;
		Warn("Writing to file %s",argv[optind]) ;
	}

	
	if(needle_stdin==TRUE) {

		if((inseq=ReadSeqFrag(in,0,0,0))==NULL) {
			Error("Could not read database file from input file") ;
		}
		
		while((needle=ReadSeqFrag(query,0,0,0))!=NULL) {
			if(debug) Warn("qs: %s vs. %s",needle->name,inseq->name) ;
			SearchNeedle(out,inseq,needle) ;
			free(needle) ;
		}


	} else {
		while((inseq=ReadSeqFrag(in,0,0,0)) != NULL) {
			SearchNeedle(out,inseq,needle) ;
			free(inseq) ;
		}
	}

	

	if(html) PrintAllWarnings(out) ;
	
	fclose(out) ;
	fclose(in) ;
	return(0);
}

int SearchNeedle(FILE *out, sekw* s, sekw* n) {

	long dls, dln, pos1, i,j ;
	char name[31] ;
	sekw *rn ;

	for(i=0;i<10;i++) {
		name[i]=n->name[i] ;
		if(n->name[i]=='\0') i=10 ; 
	}

	name[10]='\0' ;

	strcat(name," v. ") ;

	j=strlen(name) ;
	for(i=j;i<(j+10);i++) {
		name[i]=s->name[(i-j)] ;
		if(s->name[(i-j)]=='\0') i=j+10 ;
	}

	name[j+10]='\0' ;

	dls=strlen(s->sequ) ;
	dln=strlen(n->sequ) ;

	/* preparing the reverse needle */
	rn=ReverseSequence(n) ;

	/* The main search loop */
	for(pos1=0;pos1<(dls-dln);pos1++) {
		
		if(plus) {
			i=0 ;
			while(i<dln && n->sequ[i]==s->sequ[pos1+i]) i++ ;
			if(i==dln)
				fprintf(out,"%li %li E %s\n", pos1+1,pos1+dln,name) ;
			else if(degenerate==TRUE) {
				i=varia ;
				while(i<(dln-varia) && n->sequ[i]==s->sequ[pos1+i]) i++ ;
				if(i==(dln-varia))
					fprintf(out,"%li %li N %s\n", pos1+varia+1,pos1+dln-varia,name) ;
			}
		}

		if(minus) {
			i=0 ;
			while(i<dln && rn->sequ[i]==s->sequ[pos1+i]) i++ ;
			if(i==dln)
				fprintf(out,"%li %li E %s\n", pos1+dln,pos1+1,name) ;
			else if(degenerate==TRUE) {
				i=varia ;
				while(i<(dln-varia) && rn->sequ[i]==s->sequ[pos1+i]) i++ ;
				if(i==(dln-varia))
					fprintf(out,"%li %li N %s\n", pos1+dln-varia,pos1+varia+1,name) ;
			}
		}




	}

	free(rn) ;

}

/* Standard mesage */

void Help()
{
printf("\n");
printf("%s version %s - quick sequence search",PROGNAME,VERSION);
printf("\n");
printf("  Usage:\n");
printf("     %s [options] sequence [ database file ] [ output file ]\n",progname);
printf("     %s -i [options] database file [ output file ]\n",progname);
printf("\n");
printf("  Options:\n");
printf("     -i       : query sequences are read from standard input\n");
printf("     -n       : allow not exact binding\n");
printf("     -v       : print version information & exit\n");
printf("     -h       : print this help screen & exit\n");
printf("     -q       : quiet; suppress error messages\n\n");
exit(0);
}


			
