/* ----------------------------------------------------------------------
 * qs -- quick sequence search
 * Copyright (C) 2000 January Weiner III
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 ---------------------------------------------------------------------- */

#include <stdio.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>

#include "myheaders.h"

#define VERSION "0.1"
#define PROGNAME "gp_qs"
#define VARIA 2

int plus=TRUE,  minus=TRUE, degenerate=FALSE ;
int varia=VARIA ;
char *progname ;

int SearchNeedle(FILE *out,sekw* s, sekw* needle) ;

/*
 *
 */

int main(int argc, char *argv[])
{
	extern int optind ;
	extern char *optarg ;
	position_s *result, *temp ;

	FILE *in, *out, *query ;
	sekw *inseq ;
	sekw *needle ;

	int c, needle_stdin=FALSE;
	char message[100] ;
	int errflg = 0 ;

	allwarnings=NULL ;

	progname=argv[0] ;

	while ((c = getopt(argc, argv, "nimpqvdh")) != EOF)
		switch(c) {
		case 'n':
			Warn("Allowing not exact binding, %i",varia) ;
			degenerate=TRUE ;
			break ;
		case 'i':
			Warn("Reading query sequences from standard input") ;
			needle_stdin=TRUE ;
			break ;
		case 'm':
			Warn("Searching only on the minus strang") ;
			plus=FALSE ;
			break ;
		case 'p':
			Warn("Searching only on the plus strang") ;
			minus=FALSE ;
			break ;
		case 'q':
			quiet=TRUE ;
			break ;
		case 'v':
			fprintf(stderr,"%s version %s\n",progname,VERSION) ;
			exit(0) ;
			break ;
		case 'H':
			html=TRUE ;
			break ;
		case 'd':
			debug=TRUE ;
			Warn("Running in debug mode") ;
			break ;
		case 'h':
			break ;
		default:
			errflg++ ;
			break;
		}

	if(optind >= argc) {
		Warn("Not enough parameters given") ;
		errflg++ ;
	}

	if(errflg) {
			sprintf(message,"Type '%s -h' for help",progname) ;
			Error(message) ;
	}

	/* 
	 * if option -i defined, then reading sequences from stdin 
	 * and the database sequence from a file. 
	 * otherwise, try to get the sequences from command line 
	 * and the database sequence either from a file, or stdin
	 */

	if(needle_stdin==FALSE) {
		needle=(sekw*) calloc(1,sizeof(sekw)) ;
		needle->sequ=(char *) calloc(strlen(argv[optind])+1,sizeof(char)) ;
		strcpy(needle->sequ,argv[optind]) ;
		strcpy(needle->name,"Unknown") ;
		optind++ ;
		if(debug) Warn("Query: %s",needle->sequ) ;
	} else {
		query=stdin ;
		if(debug) Warn("Reading query from stdandard input") ;
	}

	

	/* open the file pointer to read the sequences 
	 * from: standard input or a file provided? */
	if(optind >= argc) {
		if(needle_stdin==TRUE) 
			Error("Cannot read both query and database from standard input!") ;
		in=stdin ;
	} else {
		if(debug) Warn("Reading database from file %s",argv[optind]) ;
		in=TryOpen(argv[optind],"r") ;
		optind++ ;
	}


/* opening the file pointer to write the output: 
 * standard output or file provided? */

	if(optind >= argc) {
		if(debug) Warn("Writing to standard output") ;
		out=stdout ;
	} else {
		out=TryOpen(argv[optind],"wb") ;
		if(debug) Warn("Writing to file %s",argv[optind]) ;
	}

	
	if(needle_stdin==TRUE) {

		if((inseq=ReadSeqFrag(in,0,0,0))==NULL) {
			Error("Could not read database file from input file") ;
		}
		
		while((needle=ReadSeqFrag(query,0,0,0))!=NULL) {
			if(debug) Warn("qs: %s vs. %s",needle->name,inseq->name) ;
			result=FindSeq(needle,inseq) ;
			while(result!=NULL) {
				fprintf(out,"%li %li\n",result->start,result->end) ;
				temp=(position_s*) result->next ;
				free(result) ;
				result=temp ;
			}
			free(needle) ;
		}


	} else {
		while((inseq=ReadSeqFrag(in,0,0,0)) != NULL) {
			result=FindSeq(needle,inseq) ;
			while(result!=NULL) {
				fprintf(out,"%li %li\n",result->start,result->end) ;
				temp=(position_s*) result->next ;
				free(result) ;
				result=temp ;
			}

			free(inseq) ;
		}
	}

	

	if(html) PrintAllWarnings(out) ;
	
	fclose(out) ;
	fclose(in) ;
	return(0);
}

